package Kubernetes::REST::Call::v1beta2::Apps::WatchDeploymentListForAllNamespaces;
our $VERSION = '1.104';
# ABSTRACT: Deprecated - use the new Kubernetes::REST API instead
use strict;
use warnings;
warn __PACKAGE__ . " is deprecated, use the new Kubernetes::REST API instead";
1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Kubernetes::REST::Call::v1beta2::Apps::WatchDeploymentListForAllNamespaces - Deprecated - use the new Kubernetes::REST API instead

=head1 VERSION

version 1.104

=head1 SUPPORT

=head2 Issues

Please report bugs and feature requests on GitHub at
L<https://github.com/pplu/kubernetes-rest/issues>.

=head2 IRC

Join C<#kubernetes> on C<irc.perl.org> or message Getty directly.

=head1 CONTRIBUTING

Contributions are welcome! Please fork the repository and submit a pull request.

=head1 AUTHORS

=over 4

=item *

Torsten Raudssus <torsten@raudssus.de>

=item *

Jose Luis Martinez Torres <jlmartin@cpan.org>

=back

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2019-2026 by Jose Luis Martinez Torres <jlmartin@cpan.org>.

This is free software, licensed under:

  The Apache License, Version 2.0, January 2004

=cut
